#!/usr/bin/env python3

import nbformat
import unittest
import re
from pathlib import Path

from tmc import points
from tmc.applt import write_result, prepare_cell, Colors

# Define path to Notebook
nb_path = 'grade_my_work.ipynb'

# Read the notebook
nb = nbformat.read(nb_path, as_version=4)

# Collect indices for notebook cells that contain exercises. Map these indices
# to a dictionary keyed by test names.
exercises = {cell['metadata']['test']: ix for ix, cell in enumerate(nb.cells)
             if 'test' in cell['metadata']}

# Check if the course should be graded
if 'grade' in exercises:

    # Run grade cell
    exec(nb.cells[exercises['grade']].source)

    if locals()['grade'] in [False, None]:

        grade = False

    else:

        grade = True


class CheckGrading(unittest.TestCase):

    @classmethod

    def setUpClass(cls):

        # Prepare the cell by clearing outputs
        prepare_cell(nb.cells[exercises['grade']])

    def setUp(self):

        # Get the cell that is to be tested
        self.cell = nb.cells[exercises['grade']]

    def test_grade_true(self):

        # Perform test
        try:

            # Run the cell within the function for local scope
            exec(self.cell.source)

            # Check condition
            assert locals()['grade']

            # Define message for passing the test
            message = {'text/plain': f'{Colors.BOLD}[TMC]{Colors.ENDC} '
                                     f'{Colors.OKGREEN}Your coursework is set to be graded. '
                                     f'Please submit this Notebook to the TMC '
                                     f'server to notify the instructor.{Colors.ENDC}'}

            # Set flag
            fail = False

        except AssertionError:

            # Define error message
            errmsg = 'Your coursework is not set to be graded. Please set the variable "grade" ' \
                     'to True if you wish to have your coursework graded.'

            # Define a message for failing the test
            message = {'text/plain': f'{Colors.BOLD}[TMC]{Colors.ENDC} '
                                     f'{Colors.FAIL}{errmsg}{Colors.ENDC}'}

            # Set flag
            fail = True

        # Write the result to the cell
        write_result(nb=nb,
                     nb_path=nb_path,
                     cell=self.cell,
                     output_type='execute_result',
                     data=message,
                     execution_count=self.cell.execution_count)

        # Check flag
        if fail:

            # Use the fail method to fail the test
            self.fail(errmsg)

